#  黄道吉日
#  (c) シオビ	http://www.siobi.info/

#  日時や場所から、無限遠光源の位置を計算・設定します。
#　緯度をマイナスにすると南緯になり、経度をマイナスにすると西経になります。
#
#  2009/09/09 (ver. 3.10) UUIDの利用を中止（設定が正しく保存されないため）。ダイアログを改良。
#  2009/09/08 (ver. 3.00) 設定した際のメッセージを追加・修正。UUIDを利用。初期値を修正。Python版完成。
#  2009/09/01 (ver. ----) 後半を移植。
#  2009/07/16 (ver. ----) 前半を移植。
#  2009/07/15 (ver. ----) Pythonを作成開始。
#  2002/09/21 (ver. 1.00) Tcl版作成。[2002/09/21～2006/02/18]


SCRIPT_ID = 0x51081A00


# 関数
from math import *

def floor(number):
	result = 0
	while result < number:
		result = result+1
	result = result-1
	return result

def sundial(scene):

	# ■ダイアログの作成
	dialog = xshade.create_dialog(SCRIPT_ID)
#	dialog = xshade.create_dialog_with_uuid('51081A00-2003-0903-0409-51081B000000')

#	dialog.begin_box()
	dialog.begin_group('日時')
	index_year  = dialog.append_int("", "年")
	index_month = dialog.append_int("", "月")
	index_day   = dialog.append_int("", "日")
	index_hour  = dialog.append_int("", "時")
	index_min   = dialog.append_int("", "分")
	dialog.end_group() 
	dialog.begin_group('場所')
	index_lat   = dialog.append_float("緯度", "°")
	index_long  = dialog.append_float("経度", "°")
	index_LT    = dialog.append_float("標準時との時差", "時間")
	dialog.end_group() 
	dialog.begin_group('Shade') 
	index_V     = dialog.append_float("北の向き", "°")
	dialog.end_group()
	dialog.append_default_button() 
#	dialog.end_box()


	# 初期値
	dialog.set_default_value(index_year, 2009)
	dialog.set_default_value(index_month, 7)
	dialog.set_default_value(index_day, 22)
	dialog.set_default_value(index_hour, 10)
	dialog.set_default_value(index_min, 00)
	dialog.set_default_value(index_lat, 35.6984)
	dialog.set_default_value(index_long, 139.4139)
	dialog.set_default_value(index_LT, 9)
	dialog.set_default_value(index_V, 0.00)

	if not dialog.ask("黄道吉日"):
		return False

	# 値の取得
	Y		= dialog.get_value(index_year)
	M		= dialog.get_value(index_month)
	D	    = dialog.get_value(index_day)
	hour	= dialog.get_value(index_hour)
	min	    = dialog.get_value(index_min)
	LAT	    = dialog.get_value(index_lat)
	LONG	= dialog.get_value(index_long)
	LT		= dialog.get_value(index_LT)
	V		= dialog.get_value(index_V)


	# ●計算
	# 　参考文献：長沢 工著 「日の出・日の入りの計算」 地人書館。
	# 　一部の計算間違い(?)を手直し済み。
	try:
	   # 0.準備
		pi = 3.14159265
		sec = 0.00
#		V = 0.00


	   # 1.時刻変数
		Kd = 365*(Y-2000) + 30*M + D-33.5 + LT/24.0 + floor(3*(M+1)/5.0) + floor((Y-2000)/4.0)	# K'
		dT = 64 + (Y-2000)*14/20.0																# ΔT
		K  = Kd + (hour + min/60.0 + sec/3600.0)/24.0 + dT/86400.0								# J2000.0からの経過日数
		T  = K/365.25																			# J2000.0からの経過ユリウス年


	   # 2.太陽の視黄経
		lambda_s = ( 280.4603 + 360.00769*T
		+(1.9146-0.00005*T)*sin((357.538+359.991*T)*pi/180.0)
					+0.0200*sin((355.05+719.981*T)*pi/180.0)
					+0.0048*sin((234.95+ 19.341*T)*pi/180.0)
					+0.0020*sin((247.1 + 329.64*T)*pi/180.0)
					+0.0018*sin((297.8 +4452.67*T)*pi/180.0)
					+0.0018*sin((251.3 +   0.20*T)*pi/180.0)
					+0.0015*sin((343.2 + 450.37*T)*pi/180.0)
					+0.0013*sin(( 81.4 + 225.18*T)*pi/180.0)
					+0.0008*sin((132.5 + 659.29*T)*pi/180.0)
					+0.0007*sin((153.3 +  90.38*T)*pi/180.0)
					+0.0007*sin((206.8 +  30.35*T)*pi/180.0)
					+0.0006*sin(( 29.8 + 337.18*T)*pi/180.0)
					+0.0005*sin((207.4 +   1.50*T)*pi/180.0)
					+0.0005*sin((291.2 +  22.81*T)*pi/180.0)
					+0.0004*sin((234.9 + 315.56*T)*pi/180.0)
					+0.0004*sin((157.3 + 299.30*T)*pi/180.0)
					+0.0004*sin(( 21.1 + 720.02*T)*pi/180.0)
					+0.0003*sin((352.5 +1079.97*T)*pi/180.0)
					+0.0003*sin((329.7 +  44.43*T)*pi/180.0) )

		lambda_s = lambda_s - floor(lambda_s/360.0)*360.0


	   # 3.太陽の赤経・赤緯への換算
		epsilon = 23.439291-0.000130042*T								# 黄道傾角 ε
		alpha = atan(tan(lambda_s*pi/180.0)*cos(epsilon*pi/180.0))		# 赤経	   α
		delta = asin(sin(lambda_s*pi/180.0)*sin(epsilon*pi/180.0))		# 赤緯	   δ

		if lambda_s >= 90.0 and lambda_s < 270.0:
			alpha = alpha*180/pi+180.0
		else:
			alpha = alpha*180/pi


	   # 4.恒星時
		dD	  = (hour + min/60.0 + sec/3600.0)/24.0												#
		Theta = 100.4606 + 360.007700536*T + 0.00000003879*T*T + 360.0*dD - 15.0*LT + LONG		# 恒星時 Θ
		Theta = Theta - floor(Theta/360.0)*360.0												#


	   # 5.太陽の方位角、高度
		t = (Theta-alpha)*pi/180.0																# 時角	 t
		denominator = sin(delta)*cos(LAT*pi/180.0)-cos(delta)*sin(LAT*pi/180.0)*cos(t)
		if denominator != 0.0:
			A = atan(-cos(delta)*sin(t)/denominator)											# 方位角 A

		h = asin(sin(delta)*sin(LAT*pi/180.0)+cos(delta)*cos(LAT*pi/180.0)*cos(t))				# 高度	 h
		h = h*180.0/pi

		sin_t = sin(t)
		if denominator < 0:
			A = A + pi
		elif denominator == 0:
			if sin_t > 0:
				A = pi*(-0.5)
			elif sin_t < 0:
				A = pi*0.5
			else:
				A = 0.0
				h = 90.0

		A = A+V*pi/180.0		# 北の向きに向ける。（0°ならZ軸の負の方向が北。90°ならX軸の正の方向。）


	   # 6.大気差
		h = h + 0.0167/tan((h+8.6/(h+4.4))*pi/180.0)


	   # 直交座標に変換
		Sx = cos(h*pi/180.0)*sin(A)
		Sy = sin(h*pi/180.0)
		Sz = cos(h*pi/180.0)*cos(A)*(-1)

	   # 無限遠光源の方向を設定
		DistantLightIndex = xshade.scene().distant_light.active_item
		xshade.scene().distant_light.distant_light_item(DistantLightIndex).direction = (Sx, Sy, Sz)


	   # メッセージ
		print ""
		print "入力"
		print "　日時： %d 年　%d 月　%d 日　%d 時　%d 分" % (Y, M, D,hour, min)
		print "　場所： 緯度 %+10.4f °　経度 %+10.4f °　標準時との時差 %+3.1f 時間" % (LAT, LONG, LT)
		print "　北の向き： %10.2f °" % V
		print "計算結果"
		A = A*180/pi
		print "　方位角： %10.2f °" % A
		print "　太陽高度： %10.2f °" % h
		DistantLightIndex = DistantLightIndex + 1
		print "　無限円光源の番号： %d" % DistantLightIndex
		print "設定しました。"


	except:
		print "予期せぬエラーが発生しました"


can_import = True
try:
	import random
except:
	print "Pythonがインストールされていません"
	can_import = False


# 関数の実行
if can_import:
	sundial(xshade.scene())
